﻿------------------------------------------------------------------------------
-- $Id$
------------------------------------------------------------------------------
--                            module_arms.lua
--
--    Copyright (C) 2014 jlongwell
--
--  This program is free software: you can redistribute it and/or modify
--  it under the terms of the GNU Lesser General Public License as published
--  by the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU Lesser General Public License
--  along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--  This file is for use with clcInfo by abija.
--
--  abija has distributed clcInfo All Rights Reserved.  For more information
--  go to: http://wow.curseforge.com/addons/clctracker/
--
------------------------------------------------------------------------------
-- Author:
--      jlongwell (Andërs@Sargeras)
------------------------------------------------------------------------------
-- Description:
--   Spec ability implementation for clcInfo_ArmsWarrior. 
------------------------------------------------------------------------------

if not clcInfo then return end       -- need clcInfo to run
if not clcInfo_Warrior then return end -- need clcInfo_Warrior
local _, englishClass = UnitClass("player") -- need to be a warrior
local fury = "Fury"
local _,specname = GetSpecializationInfo(GetActiveSpecGroup())
if englishClass ~= clcInfo_Warrior.CLASS and specname ~= fury then return end

-- Get local copies of functions/tables.  This will speed up execution.
local clcInfo_Warrior = clcInfo_Warrior
local spells = clcInfo_Warrior.spells
local GetTimeLeft = clcInfo_Warrior.GetTimeLeft
local GetCooldown = clcInfo_Warrior.GetCooldown
local IconNext = clcInfo_Warrior.IconNext
local IconCurrent = clcInfo_Warrior.IconCurrent
local CalculateTargetHealthPct = clcInfo_Warrior.CalculateTargetHealthPct
local PickShout = clcInfo_Warrior.PickShout
local UnitPower = UnitPower
local UnitAura = UnitAura
local GetNumPartyMembers = GetNumPartyMembers
local GetNumRaidMembers = GetNumRaidMembers
local DELAY = clcInfo_Warrior.DELAY
local DISCOUNT = clcInfo_Warrior.DISCOUNT

-- Assign module variables
local specname = "arms"
local prio_arg = "armsprio"
local sample_prio = "ms cs exe csrefresh op slam shout"
local priority = {"ms","cs","exe","csfresh","op","slam","shout"}
local class_module
local class_module_name

local db_default = {
    char_var = false,
}
local db

local tdb_default = {
    version = clcInfo_Warrior.VERSION,
    priority = priority,
}
local tdb

-- actions ---------------------------------------------------------------------
-- local copies of spell names and ids to ugly up code but speed action
-- functions up
local MortalStrikeId = spells["Mortal Strike"].id
local BattleShoutId = spells["Battle Shout"].id
local ColossusSmashId = spells["Colossus Smash"].id
local CommandingShoutId = spells["Commanding Shout"].id
local ExecuteId = spells["Execute"].id
local HeroicStrikeId = spells["Heroic Strike"].id
local OverpowerId = spells["Overpower"].id
local SlamId = spells["Slam"].id

local Enrage = spells["Enrage"].name -- Wrecking Crew buff
local MortalStrike =  spells["Mortal Strike"].name
local TasteforBlood = spells["Taste for Blood"].name
local BattleShout = spells["Battle Shout"].name
local ColossusSmash = spells["Colossus Smash"].name
local CommandingShout = spells["Commanding Shout"].name
local Execute = spells["Execute"].name
local Overpower = spells["Overpower"].name
local Slam = spells["Slam"].name

local actions = {
    ms = {
        id = function()
            return MortalStrikeId
        end,
        GetCD = function(cur_id, rage)
            if cur_id ~= MortalStrikeId then
                local _, _, _, cost = GetSpellInfo(MortalStrikeId)
                local add_time = ((cost > rage) and DELAY) or 0
                return GetCooldown(MortalStrikeId) + add_time
            end
            return DISCOUNT
        end,
        info = spells["Mortal Strike"].name,
    },
    cs = {
        id = function()
            return ColossusSmashId
        end,
        GetCD = function(cur_id, rage)
            if cur_id ~= ColossusSmashId
               and GetTimeLeft("target",ColossusSmash,"HARMFUL|PLAYER")==0 then
                local _, _, _, cost = GetSpellInfo(ColossusSmashId)
                local add_time = ((cost > rage) and DELAY) or 0
                return GetCooldown(ColossusSmashId) + add_time
            end
            return DISCOUNT
        end,
        info = spells["Colossus Smash"].name,
    },
    op = {
        id = function()
            return OverpowerId
        end,
        GetCD = function(cur_id, rage)
            if cur_id ~= OverpowerId and
               not (GetTimeLeft("player","Recklessness","HELPFUL|PLAYER") > .5)
               and (GetCooldown(MortalStrikeId)>.5 or cur_id==MortalStrikeId)
               and (UnitPower('player') < 80)
               and not (GetTimeLeft("player","sweeping Strikes","HELPFUL|PLAYER") > .5)
			then
                local _, _, _, cost = GetSpellInfo(OverpowerId)
                local add_time = ((cost > rage) and DELAY) or 0
                return GetCooldown(OverpowerId) + add_time
            end
            return DISCOUNT
        end,
        info = spells["Overpower"].name,
    },
    exe = {
        id = function()
            return ExecuteId
        end,
        GetCD = function(_, rage)
            if CalculateTargetHealthPct() <= 20 then
                local _, _, _, cost = GetSpellInfo(ExecuteId)
                local add_time = ((cost > rage) and DELAY) or 0
                return GetCooldown(ExecuteId) + add_time
            end
            return DISCOUNT
        end,
        info = spells["Execute"].name,
    },
    csrefresh = {
        id = function()
            return ColossusSmashId
        end,
        GetCD = function(cur_id, rage)
            if cur_id ~= ColossusSmashId then
                local _, _, _, cost = GetSpellInfo(ColossusSmashId)
                local add_time = ((cost > rage) and DELAY) or 0
                return max(GetTimeLeft("target",ColossusSmash,"HARMFUL|PLAYER")-2,
                           GetCooldown(ColossusSmashId))
                       + add_time
            end
            return DISCOUNT
        end,
        info = spells["Colossus Smash"].name..": "..REFRESH,
    },
    slam = {
        id = function()
            return SlamId
        end,
        GetCD = function(_, rage)
            if ((rage >= 80  or GetTimeLeft("player", "Recklessness", "HELPFUL") > .5) and GetTfB() < 4)
			or GetTimeLeft("player", "Sweeping Strikes", "HELPFUL") > .5 
			then
                local _, _, _, cost = GetSpellInfo(SlamId)
                local add_time = ((cost > rage) and DELAY) or 0
                return GetCooldown(SlamId) + add_time
            end
            return DISCOUNT
        end,
        info = spells["Slam"].name,
    },
    shout = {
        id = function()
            return PickShout()
        end,
        GetCD = function(cur_id, rage)
            if cur_id ~= BattleShoutId and cur_id ~= CommandingShoutId
               and rage < 60 then
                return GetCooldown(BattleShoutId)
            end
            return DISCOUNT
        end,
        info= "Shout",
    },
}

----FUNCTIONS-------------------------------------------------------------------
-- The following are called once or infrequently, so speed isn't an issue.

-- called whenever module options are loaded
local GetDB
local SetDB
local GetTDB
local SetTDB
local GetPriority
local SetPriority

function GetTfB()
	local _,_,_,stacks = UnitAura("player", "Taste for Blood")
	if stacks ~= nil then
		return stacks
	else
		return 0
	end
end
--initialize module.
class_module, class_module_name = clcInfo_Warrior:init_module(specname)

--called when template updates.
function class_module:OnTemplatesUpdate()
    tdb, GetTDB, SetTDB, GetPriority, SetPriority
        = clcInfo_Warrior:OnTemplatesUpdate(class_module_name, actions,
                                          tdb_default, specname)
end

-- called each time clcInfo initializes
function class_module.OnInitialize()
    db,GetDB,SetDB = clcInfo_Warrior:OnInitialize(class_module_name, db_default)
end

-- called when module options are opened
local function LoadArmsModule()
    local _, talent_tree_name = GetSpecializationInfo(1)
    clcInfo_Warrior.LoadModule(class_module_name, talent_tree_name, sample_prio,
                               actions, GetDB, SetDB, GetTDB, SetTDB,
                               GetPriority, SetPriority)
end
clcInfo.optionsCMLoaders[#(clcInfo.optionsCMLoaders) + 1] = LoadArmsModule

-- giving queue as command line
local function CmdArms(args)
    clcInfo_Warrior:CmdLine(tdb, args, actions, specname)
end

clcInfo.cmdList[prio_arg] = CmdArms

----ABILITY ICONS--------------------------------------------------------------
----Add these to the clcInfo.env (so they live with IconAura, etc.
local emod = clcInfo.env
function emod.IconArmsCurrent(...)
    return IconCurrent(actions, specname, tdb.rangePerSkill)
end
function emod.IconArmsNext(...)
    return IconNext()
end

-- Heroic Strike Icon
-- This is an internal icon since there are so many conditionals that
-- leveraging an alarm is smoother if we do it this way.
function emod.IconArmsHeroicStrike()
    local target_health_pct = CalculateTargetHealthPct()
    local rage = UnitPower("player")
    if rage > 85
       or (UnitAura("player", DeadlyCalm, nil, "HELPFUL"))
       or ((UnitAura("player", InnerRage, nil, "HELPFUL"))
           and ((target_health_pct > 20 and rage >= 50)
                or (target_health_pct <= 20
                    and (rage >= 50 or 
                         (UnitAura("player",BattleTrance,nil,"HELPFUL"))))))
       then
        return emod.IconSpell(HeroicStrikeId, true) 
    end
    return nil
end